Public Class Channel
    Public Title As String
    Public LastBuildDate As Date
    Public Description As String
    Public Link As String
    Public Articles() As Article

    ' Return the value we want displayed in the list.
    Public Overrides Function ToString() As String
        Return Title
    End Function

    ' Update the channel.
    Public Sub Update(ByVal new_channel As Channel)
        ' See if the new channel is more recent.
        ' (If the new channel is no more recent,
        ' we ignore it.)
        If new_channel.LastBuildDate > Me.LastBuildDate Then
            ' The new channel is more recent.
            ' Mark deleted Articles as not touched.
            For Each old_article As Article In Me.Articles
                old_article.Touched = False
            Next old_article

            ' Merge in the new articles.
            For Each new_article As Article In new_channel.Articles
                MergeArticle(new_article)
            Next new_article

            ' Remove deleted articles that were not touched.
            Dim i As Integer = 0
            Dim new_articles() As Article
            For Each old_article As Article In Me.Articles
                ' Keep this article if it is not deleted
                ' or it was touched.
                If (Not old_article.IsDeleted) Or (old_article.Touched) Then
                    ReDim Preserve new_articles(i)
                    new_articles(i) = old_article
                    i += 1
                End If
            Next old_article
            Me.Articles = new_articles
        End If
    End Sub

    ' Merge this article with the existing ones.
    Private Sub MergeArticle(ByVal new_article As Article)
        ' See if we already have this article's link.
        Dim old_article As Article = FindArticle(new_article.Link)

        If old_article Is Nothing Then
            ' We do not have this article yet.
            ' Add it to the Articles array.
            Dim max_index As Integer = Articles.Length
            ReDim Preserve Articles(max_index)
            Articles(max_index) = new_article
        Else
            ' We already have this article.
            ' Compare the two versions' dates.
            If new_article.PubDate > old_article.PubDate Then
                ' This is an update. Save its data.
                old_article.Title = new_article.Title
                old_article.Description = new_article.Description
                old_article.PubDate = new_article.PubDate
                old_article.IsRead = False
            End If

            ' If the Article is marked deleted,
            ' mark it as touched.
            If old_article.IsDeleted Then old_article.Touched = True
        End If
    End Sub

    ' Find the indicated article.
    Private Function FindArticle(ByVal target_link As String) As Article
        ' Make sure we have articles.
        If Articles Is Nothing Then Return Nothing

        ' Find the article.
        For i As Integer = 0 To Articles.GetUpperBound(0)
            If Articles(i).Link = target_link Then Return Articles(i)
        Next i

        ' We didn't find it.
        Return Nothing
    End Function
End Class
